/*
   AngelScript library
   Copyright (C) 2003 Andreas Jnsson

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



   The original version of this library can be located at:
   http://www.angelcode.com/angelscript/

   Andreas Jnsson
   andreas@angelcode.com
*/


//
// angelscript.h
//
// The script engine interface
//


#ifndef ANGELSCRIPT_H
#define ANGELSCRIPT_H

// AngelScript version

#define ANGELSCRIPT_VERSION        10601
#define ANGELSCRIPT_VERSION_MAJOR  1
#define ANGELSCRIPT_VERSION_MINOR  6
#define ANGELSCRIPT_VERSION_BUILD  1
#define ANGELSCRIPT_VERSION_STRING "1.6.1b"

// Data types

class asIScriptEngine;
class asIScriptContext;
class asIOutputStream;
typedef unsigned char * asBSTR;
typedef unsigned long  asDWORD;
typedef unsigned short asWORD;
typedef unsigned char  asBYTE;
typedef unsigned int   asUINT;

class asCDummy{};
typedef void (asCDummy::*asMETHOD_t)();
typedef void (*asFUNCTION_t)();

inline void *asForceVoidPtr(asMETHOD_t x) {return *(void**)&(x);}
inline void *asForceVoidPtr(asFUNCTION_t x) {return *(void**)&(x);}

#define asMETHOD(x) asForceVoidPtr(asMETHOD_t(x))
#define asFUNCTION(x) asForceVoidPtr(asFUNCTION_t(x))

// API functions

// ANGELSCRIPT_EXPORT is defined when compiling the dll or lib
// ANGELSCRIPT_DLL_LIBRARY_IMPORT is defined when dynamically linking to the 
// dll through the link lib automatically generated by MSVC++
// ANGELSCRIPT_DLL_MANUAL_IMPORT is defined when manually loading the dll
// Don't define anything when linking statically to the lib

#ifdef WIN32
  #ifdef ANGELSCRIPT_EXPORT
    #define AS_API __declspec(dllexport)
  #elif defined ANGELSCRIPT_DLL_LIBRARY_IMPORT
    #define AS_API __declspec(dllimport)
  #else // statically linked library
    #define AS_API 
  #endif
  #define AS_CALL __stdcall
#else
  #define AS_API
  #define AS_CALL
#endif

#ifndef ANGELSCRIPT_DLL_MANUAL_IMPORT
extern "C"
{
	// Engine
	AS_API asIScriptEngine * AS_CALL asCreateScriptEngine(asDWORD version);
	AS_API const char * AS_CALL asGetLibraryVersion();

	// Context
	AS_API asIScriptContext * AS_CALL asGetActiveContext();

	// String handling
	AS_API asBSTR AS_CALL asBStrAlloc(asUINT length);
	AS_API void   AS_CALL asBStrFree(asBSTR str);
	AS_API asUINT AS_CALL asBStrLength(asBSTR str);
}
#endif

// Interface declarations

class asIScriptEngine
{
public:
	// Memory management
	virtual int AS_CALL AddRef() = 0;
	virtual int AS_CALL Release() = 0;

	// Engine configuration
	virtual int AS_CALL RegisterObjectType(const char *name, int byteSize, asDWORD flags) = 0;
	virtual int AS_CALL RegisterObjectProperty(const char *obj, const char *declaration, int byteOffset, asDWORD flags) = 0;
	virtual int AS_CALL RegisterObjectMethod(const char *obj, const char *declaration, void *funcPointer, int callConv) = 0;

	virtual int AS_CALL RegisterGlobalProperty(const char *declaration, void *pointer, asDWORD flags) = 0;
	virtual int AS_CALL RegisterGlobalFunction(const char *declaration, void *funcPointer, int callConv) = 0;

	virtual int AS_CALL RegisterTypeBehaviour(const char *datatype, asDWORD behaviour, const char *declaration, void *funcPointer, int callConv) = 0;

	// Script building
	virtual int AS_CALL AddScriptSection(const char *name, const char *code, int codeLength) = 0;
	virtual int AS_CALL Build(asIOutputStream *out) = 0;

	virtual int AS_CALL GetFunctionCount() = 0;
	virtual int AS_CALL GetFunctionIDByName(const char *name) = 0;
	virtual int AS_CALL GetFunctionIDByDecl(const char *decl) = 0;
	virtual int AS_CALL GetFunctionDeclaration(int funcID, char *buffer, int bufferSize) = 0;
	virtual int AS_CALL GetFunctionName(int funcID, char *buffer, int bufferSize) = 0;

	// Script execution
	virtual int AS_CALL CreateContext(asIScriptContext **context) = 0;
};

class asIScriptContext
{
public:
	// Memory management
	virtual int AS_CALL AddRef() = 0;
	virtual int AS_CALL Release() = 0;

	// Engine
	virtual asIScriptEngine * AS_CALL GetEngine() = 0;

	// Script context
	virtual int AS_CALL GetState() = 0;

	virtual int AS_CALL Prepare(int funcID, int stackSize) = 0;

	virtual int AS_CALL SetArguments(int stackPos, asDWORD *data, int count) = 0;
	virtual int AS_CALL GetReturnValue(asDWORD *data, int count) = 0;

	virtual int AS_CALL Execute() = 0;
	virtual int AS_CALL ExecuteStep() = 0;
	virtual int AS_CALL Abort() = 0;
	virtual int AS_CALL Suspend() = 0;

	virtual int AS_CALL GetCurrentLineNumber() = 0;
	virtual int AS_CALL GetCurrentFunction() = 0;

	// Exception handling
	virtual int AS_CALL SetException(const char *string) = 0;
	virtual int AS_CALL GetExceptionLineNumber() = 0;
	virtual int AS_CALL GetExceptionFunction() = 0;
	virtual int AS_CALL GetExceptionString(char *buffer, int bufferSize) = 0;
};

class asIOutputStream
{
public:
	virtual void AS_CALL Write(const char *text) = 0;
};

// Enumerations and constants

// Calling conventions and flags

#define asCALL_CDECL         0
#define asCALL_THISCALL      1
#define asCALL_CDECL_OBJLAST 2
#define asCALL_RETURNBYVAL 0x100
#define asCALL_RETURNBYREF 0x200

// Behaviours

#define asBEHAVE_CONSTRUCT     0
#define asBEHAVE_DESTRUCT      1
#define asBEHAVE_ASSIGNMENT    2
#define asBEHAVE_FIRST_DUAL    3
 #define asBEHAVE_ADD           3
 #define asBEHAVE_SUBTRACT      4
 #define asBEHAVE_MULTIPLY      5
 #define asBEHAVE_DIVIDE        6
 #define asBEHAVE_MODULO        7
 #define asBEHAVE_EQUAL         8
 #define asBEHAVE_NOTEQUAL      9
 #define asBEHAVE_LESSTHAN      10
 #define asBEHAVE_GREATERTHAN   11
 #define asBEHAVE_LEQUAL        12
 #define asBEHAVE_GEQUAL        13
 #define asBEHAVE_LOGIC_OR      14
 #define asBEHAVE_LOGIC_AND     15
 #define asBEHAVE_BIT_OR        16
 #define asBEHAVE_BIT_AND       17
 #define asBEHAVE_BIT_XOR       18
 #define asBEHAVE_BIT_SLL       19
 #define asBEHAVE_BIT_SRL       20
 #define asBEHAVE_BIT_SRA       21
#define asBEHAVE_LAST_DUAL     21

// Return codes

#define asSUCCESS				 0
#define asERROR					-1
#define asCONTEXT_ACTIVE        -2
#define asCONTEXT_NOT_FINISHED  -3
#define asCONTEXT_NOT_PREPARED  -4
#define asINVALID_ARG           -5
#define asNO_FUNCTION           -6
#define asNOT_SUPPORTED         -7
#define asINVALID_NAME          -8
#define asNAME_TAKEN            -9
#define asINVALID_DECLARATION   -10
#define asINVALID_OBJECT        -11
#define asINVALID_TYPE          -12
#define asALREADY_REGISTERED    -13

// Context states

#define asEXECUTION_FINISHED	  0
#define asEXECUTION_SUSPENDED	  1
#define asEXECUTION_ABORTED		  2
#define asEXECUTION_EXCEPTION	  3
#define asEXECUTION_PREPARED      4
#define asEXECUTION_UNINITIALIZED 5
#define asEXECUTION_ACTIVE        6

#endif
