#include "scriptengine.h"
#include "main.h"

/**
 * Constructor
**/
CScriptEngine::CScriptEngine()
{
	engine = 0x0;
	ctx = 0x0;
}

/**
 * Initialise the angelscript engine and register all the classes, methods and functions that will be exposed to scripts
 *
 * @return		true on a successful initialisation
**/
bool CScriptEngine::init() 
{
	int r;
	engine = asCreateScriptEngine(ANGELSCRIPT_VERSION);
	
	if( engine == 0 )
	{
		printf("Failed to create script engine - Expecting library version %s, found %s\n", ANGELSCRIPT_VERSION_STRING, asGetLibraryVersion());
		return false;
	}
	
	// Register classes and methods
	r = engine->RegisterObjectType("CItem", sizeof(CItem), 0); assert(r >= 0);
	r = engine->RegisterObjectType("CPlayer", sizeof(CPlayer), 0); assert(r >= 0);
	
	r = engine->RegisterObjectProperty("CPlayer", "float health", offsetof(CPlayer, health), 0); assert(r >= 0);

	r = engine->RegisterObjectMethod("CItem", "void setFloat(bstr, float)", asMETHOD(CItem::setFloat), asCALL_THISCALL); assert (r >= 0);
	r = engine->RegisterObjectMethod("CItem", "float getFloat(bstr)", asMETHOD(CItem::getFloat), asCALL_THISCALL); assert (r >= 0);
	r = engine->RegisterObjectMethod("CItem", "void registerFloat(bstr)", asMETHOD(CItem::registerFloat), asCALL_THISCALL); assert (r >= 0);

	r = engine->RegisterGlobalFunction("void registerItemType(bstr)", asFUNCTION(CItem::registerItemType), asCALL_CDECL); assert(r >= 0);
	
	return true;
}

/**
 * Calls a function that has been defined in script.
 * There is currently no checking to verify that the arguments passed match those required by the function
 *
 * @param function		The name of the function to be called
 * @param args			A list of the arguments to be passed to the function. If null, no arguments are passed.
 * @param retVal		If this is not null, the value returned by the function is placed in the variable referenced
 * @return				true if the requested function exists
**/
bool CScriptEngine::callScript(string function, list<asDWORD *> *args, asDWORD *retVal)
{
	list<asDWORD *>::iterator it;
	int count = 0;
	int id;

	if (engine && ctx) {
		id = engine->GetFunctionIDByName(function.c_str());
		if (id < 0) {
			printf("Script function not found: %s\n", function.c_str());
			return false;
		}

		if (ctx->Prepare(id, 1000) < 0)
			return false;

		if (args)
			for ( it = args->begin(); it != args->end(); it++ ) 
				if (ctx->SetArguments(count++, *it, 1) < 0)
					printf("Definition of '%s' does not allow for enough parameters\n", function.c_str());

				
	  
		ctx->Execute();

		if (retVal)
			ctx->GetReturnValue(retVal, 1);

		return true;
	} else {
		printf("Cannot call a script function until the script engine is initialised\n");
		return false;
	}
}

/**
 * Calls a function that has been defined in script by ID. Not sure if this will be much faster.
 * There is currently no checking to verify that the arguments passed match those required by the function
 *
 * @param function		The id of the function to be called
 * @param args			A list of the arguments to be passed to the function. If null, no arguments are passed.
 * @param retVal		If this is not null, the value returned by the function is placed in the variable referenced
 * @return				true if the requested function exists
**/
bool CScriptEngine::callScript(int id, list<asDWORD *> *args, asDWORD *retVal)
{
	list<asDWORD *>::iterator it;
	int count = 0;

	if (engine && ctx) {
		if (id < 0) {
			printf("Script function id not found: %i\n", id);
			return false;
		}

		if (ctx->Prepare(id, 1000) < 0)
			return false;

		if (args)
			for ( it = args->begin(); it != args->end(); it++ ) 
				if (ctx->SetArguments(count++, *it, 1) < 0)
					printf("Definition of '%i' does not allow for enough parameters\n", id);

		ctx->Execute();

		if (retVal)
			ctx->GetReturnValue(retVal, 1);

		return true;
	} else {
		printf("Cannot call a script function until the script engine is initialised\n");
		return false;
	}
}

/**
 * Loads a script from a file and compiles it
 * This will later be extended to load pre-compiled files
 *
 * @param filename		The file to load
 * @return				true on a successful load and compile
**/
bool CScriptEngine::loadScript(string filename)
{
	char* buf;
	COutStream out;

	// Read the entire script file
	FILE *f = fopen(filename.c_str(), "rb");
	if( f == 0 )
	{
		printf("Failed to open the script file: %s\n", filename.c_str());
		return false;
	}

	int len = _filelength(_fileno(f));

	buf = new char[len];

	int c =	fread(buf, len, 1, f);
	fclose(f);

	if( c == 0 ) 
	{
		printf("Failed to open the script file: %s\n", filename.c_str());
		return false;
	}

	// Give the code to the script engine
	int r = engine->AddScriptSection(filename.c_str(), buf, len);
	if( r < 0 ) 
	{
		printf("An error occured while adding the script section: %s\n", filename.c_str());
		return false;
	}

	// At this point the engine has copied the code to an 
	// internal buffer so we are free to release the memory we 
	// allocated. 

	// We can also add other script sections if we would like.
	// All script sections will be compiled together as if they
	// where one large script. 
	delete buf;
	
	r = engine->Build(&out);
	if( r < 0 )
	{
		printf("Failed to compile script: %s\n", filename.c_str());
		return false;
	}
	if (ctx) ctx->Release();
	return (engine->CreateContext(&ctx) == asSUCCESS);
}